<?php

namespace Drupal\config_enforce\Form;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\FormHelperTrait;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Attached Config Enforce behaviour for config forms.
 */
abstract class AbstractEnforceForm extends ConfigFormBase {

  use FormHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      ConfigEnforce::CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);
  }

  /**
   * Make our form float above the config form to which it's attached.
   */
  protected function makeFormFloat() {
    $this->form()['#prefix'] = "<div class='config-enforce-form'>";
    $this->form()['#suffix'] = "</div>";
    $this->form()['#attached']['library'][] = 'config_enforce/config-enforce';
    return $this;
  }

  /**
   * Add a details section with information about the config form.
   */
  protected function addConfigFormInfo() {
    $form_id = $this->getContext('form_id');

    $this->form()['config_enforce'] = [
      '#type' => 'details',
      '#tree' =>TRUE,
      '#title' => $this->t('Config form: %id', ['%id' => $form_id]),
      '#open' => TRUE,
    ];

    $this->addEnforcedConfigItems();
    return $this;
  }

  protected function addEnforcedConfigItems(){
    foreach ($this->getFormConfigs() as $config) {
      $this->addEnforcedConfigItem($config);
    }
    return $this;
  }

  protected function addEnforcedConfigItem($config) {

    $this->form()['config_enforce'][$config] = [
      '#type' => 'details',
      '#title' => $this->t('Config object: @config', ['@config' => $config]),
      '#open' => TRUE,
    ];

    $config_enforce_enabled = ConfigEnforce::isEnforced($config);
    $this->form()['config_enforce'][$config]['config_enforce_enabled'] = [
      '#type' => 'item',
      '#title' => $this->t('Config enforce'),
      '#markup' => $config_enforce_enabled ? 'Enabled' : 'Disabled',
    ];

    if ($config_enforce_enabled) {

      $this->form()['config_enforce'][$config]['config_enforce_module'] = [
        '#type' => 'item',
        '#title' => $this->t('Module'),
        '#markup' => ConfigEnforce::getModule($config),
      ];

      $this->form()['config_enforce'][$config]['config_enforce_directory'] = [
        '#type' => 'item',
        '#title' => $this->t('Directory'),
        '#markup' => ConfigEnforce::getDirectory($config),
      ];

      $this->form()['config_enforce'][$config]['config_enforce_path'] = [
        '#type' => 'item',
        '#title' => $this->t('File path'),
        '#markup' => ConfigEnforce::getPath($config),
      ];

      $this->form()['config_enforce'][$config]['config_enforce_level'] = [
        '#type' => 'item',
        '#title' => $this->t('Enforcement level'),
        '#markup' => ConfigEnforce::getLevelLabel($config),
      ];

    }
    return $this;
  }

  /**
   * Return an item passed into the form from the config form to which we are attached.
   */
  protected function getContext($key = '') {
    $context = $this->formState()->getBuildInfo()['args'];
    return empty($key) ? $context : $context[$key];
  }

  /**
   * Return a list of configs embedded in the current form object.
   */
  protected function getFormConfigs() {
    return $this->getContext('configs');
  }

}
